//---------------------------------------------------------------------------
// libreria di funzioni statistiche (creata come parte di tuscopa.cpp)
//---------------------------------------------------------------------------
#include <vcl\condefs.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <assert.h>
#include <math.h>

#include "stocasti.h"

#pragma hdrstop
//---------------------------------------------------------------------------
USERES("stocasti.res");
//---------------------------------------------------------------------------
// distribuzione di probabilita oggettiva multinomiale (vedi initpriori)
// della distribuzione di 4 (quattro) carte su 3 giocatori
// per [ogni distribuzione possibile][per ogni numero di carte tra 0 e 4]
Multi priori[15][5] =
{
 {{{0,0,0},0.}, {{1,0,0},0.} , {{2,0,0},0.} , {{3,0,0},0.} , {{4,0,0},0.} } ,
 {{{0,0,0},0.}, {{0,1,0},0.} , {{1,1,0},0.} , {{2,1,0},0.} , {{3,1,0},0.} } ,
 {{{0,0,0},0.}, {{0,0,1},0.} , {{1,0,1},0.} , {{2,0,1},0.} , {{3,0,1},0.} } ,
 {{{0,0,0},0.}, {{0,0,0},0.} , {{0,2,0},0.} , {{1,2,1},0.} , {{2,2,0},0.} } ,
 {{{0,0,0},0.}, {{0,0,0},0.} , {{0,1,1},0.} , {{1,1,1},0.} , {{2,1,1},0.} } ,
 {{{0,0,0},0.}, {{0,0,0},0.} , {{0,0,2},0.} , {{1,0,2},0.} , {{2,0,2},0.} } ,
 {{{0,0,0},0.}, {{0,0,0},0.} , {{0,0,0},0.} , {{0,3,0},0.} , {{1,3,0},0.} } ,
 {{{0,0,0},0.}, {{0,0,0},0.} , {{0,0,0},0.} , {{0,2,1},0.} , {{1,2,1},0.} } ,
 {{{0,0,0},0.}, {{0,0,0},0.} , {{0,0,0},0.} , {{0,1,2},0.} , {{1,1,2},0.} } ,
 {{{0,0,0},0.}, {{0,0,0},0.} , {{0,0,0},0.} , {{0,0,3},0.} , {{1,0,3},0.} } ,
 {{{0,0,0},0.}, {{0,0,0},0.} , {{0,0,0},0.} , {{0,0,0},0.} , {{0,4,0},0.} } ,
 {{{0,0,0},0.}, {{0,0,0},0.} , {{0,0,0},0.} , {{0,0,0},0.} , {{0,3,1},0.} } ,
 {{{0,0,0},0.}, {{0,0,0},0.} , {{0,0,0},0.} , {{0,0,0},0.} , {{0,2,2},0.} } ,
 {{{0,0,0},0.}, {{0,0,0},0.} , {{0,0,0},0.} , {{0,0,0},0.} , {{0,1,3},0.} } ,
 {{{0,0,0},0.}, {{0,0,0},0.} , {{0,0,0},0.} , {{0,0,0},0.} , {{0,0,4},0.} }
};
//---------------------------------------------------------------------------
// calcola il "fattoriale limitato" tra (M) e (M-x+1)
// DEB dovrebbe restituire un integer o un long
double lfatt(int M, int x)
{
 double out;
 int i;

 assert(M >= x);
 for (i = M-x+1, out = 1.; i<=M; i++) out *= (double) i;
 
 return out;
}
//---------------------------------------------------------------------------
// calcola il fattoriale
// DEB dovrebbe restituire un integer o  un long
double fatt(int x)
{
 double out;
 int i;

 assert(x >= 0);
 switch (x)
 {
  case 0:
  	return (double) 1;
  case 1:
  	return (double) 1;
  case 2:
  	return (double) 2;
  case 3:
  	return (double) 6;
  case 4:
  	return (double) 24;
  default :
  	for (i = x, out = 1.; i>0; i--) out *= (double) i;
    break;
 }

 return out;
}
//---------------------------------------------------------------------------
// coefficente binomiale
// DEB dovrebbe restituire un int o un long
double cbin(int M, int x)
{
 return lfatt(M, x)/ fatt(x);
}
//---------------------------------------------------------------------------
// distribuzione ipergeometrica (estrazione senza reimissione con
// palle difettose, numero di palle difettose)
// M popolazione, K difettose, n campione, k numero di difettose
double iper(int M, int K, int n, int k)
{
 return  cbin(M - K, n - k) * cbin(K, k) / cbin(M , n);
}
//---------------------------------------------------------------------------
//Calcola la distribuzione multinomiale nel caso specifico della distribuzione
// delle n carte tra gli altri tre giocatori
double multi(int x1,int x2, int x3, int n)
{
 return fatt(n)/(fatt(x1)*fatt(x2)*fatt(x3)) * pow((double)1/3,n);
}
//---------------------------------------------------------------------------
// Inizializza (mette probabilita) dentro la struttura globale priori
void initpriori(void)
{
 int n, i;


 for (i=0; i < 15; i++)
  for (n=1; n<=4; n++)
   priori[i][n].p =
   	multi(priori[i][n].n[0],priori[i][n].n[1],priori[i][n].n[2],n);
}
//---------------------------------------------------------------------------
// distribuzione ipergeometrica (estrazione senza reimissione con
// palle difettose, numero di palle difettose) semplificata con k=0
// M popolazione, K difettose, n campione, k numero di difettose
double semiper(int M, int K, int n)
{
 return  cbin(M - K, n ) / cbin(M , n);
}
